/*
 * build.h
 * Automatically generated
 */
#define BUILD_HOSTNAME "powerpc64-1.ports.openbsd.org"
#define BUILD_KERNEL "7.9"
#define BUILD_MACHINE "powerpc64"
#define BUILD_OS "OpenBSD"
#define BUILD_DATE "2026-04-26 11:13:22 UTC"
#define BUILD_USER "_pbuild"

